@@ -13,8 +13,9 @@ from pypika.queries import Selectable, Table
 from pypika.terms import AggregateFunction, Criterion, Field, PyformatParameter
 
 from tesseract_olap.backend import ParamManager
-from tesseract_olap.query import (DataQuery, HierarchyField, MeasureField,
-                                  MembersQuery, RestrictionAge)
+from tesseract_olap.query import (Comparison, DataQuery, HierarchyField,
+                                  LogicOperator, MeasureField, MembersQuery,
+                                  NumericConstraint, RestrictionAge)
 from tesseract_olap.schema import MemberType, models
 
 
@@ -130,13 +131,16 @@ def dataquery_sql(query: DataQuery) -> Tuple[QueryBuilder, Dict[str, str]]:
 
         for field in query.fields_qualitative:
             for item in field.columns:
+                caster = _token_caster(item.level.key_type)
                 if len(item.members_include) > 0:
+                    members = sorted(caster(mem) for mem in item.members_include)
                     qb = qb.where(
-                        tcore.field(f"lv_{item.alias}").isin(item.members_include)
+                        tcore.field(f"lv_{item.alias}").isin(members)
                     )
                 if len(item.members_exclude) > 0:
+                    members = sorted(caster(mem) for mem in item.members_exclude)
                     qb = qb.where(
-                        tcore.field(f"lv_{item.alias}").notin(item.members_exclude)
+                        tcore.field(f"lv_{item.alias}").notin(members)
                     )
 
                 if item.time_restriction is not None:
@@ -201,6 +205,20 @@ def dataquery_sql(query: DataQuery) -> Tuple[QueryBuilder, Dict[str, str]]:
         )
         qb = qb.select(*measure_fields).distinct()
 
+        for field in query.fields_quantitative:
+            if not field.constraint1:
+                continue
+
+            column = Field(f"ag_{field.alias}", table=tgroup)
+            criterion = _get_filter_criterion(column, field.constraint1)
+            if field.constraint2:
+                criterion2 = _get_filter_criterion(column, field.constraint2)
+                if field.joint == LogicOperator.AND:
+                    criterion &= criterion2
+                elif field.joint == LogicOperator.OR:
+                    criterion |= criterion2
+            qb = qb.where(criterion)
+
         for field in query.fields_qualitative:
             if not field.is_drilldown:
                 continue
@@ -387,6 +405,24 @@ def _get_aggregate(table: Selectable, item: MeasureField) -> AggregateFunction:
         f"{item.aggregator_type}"
     )
 
+def _get_filter_criterion(field: Field, constr: NumericConstraint):
+    comparison, scalar = constr
+
+    if comparison == Comparison.GT:
+        return field.gt(scalar)
+    elif comparison == Comparison.GTE:
+        return field.gte(scalar)
+    elif comparison == Comparison.LT:
+        return field.lt(scalar)
+    elif comparison == Comparison.LTE:
+        return field.lte(scalar)
+    elif comparison == Comparison.EQ:
+        return field.eq(scalar)
+    elif comparison == Comparison.NEQ:
+        return field.ne(scalar)
+
+    raise NameError(f"Invalid criterion type: {comparison}")
+
 def _yield_drilldown_columns(field: HierarchyField, locale: str):
     """Generates pairs of (column name, column alias) for all fields related to
     a HierarchyField object.
@@ -403,3 +439,12 @@ def _yield_drilldown_columns(field: HierarchyField, locale: str):
         for propty in item.properties:
             propty_column = propty.get_key_column(locale)
             yield propty_column, propty.name
+
+def _token_caster(key_type: MemberType):
+    if key_type in (MemberType.INT8, MemberType.INT16, MemberType.INT32, MemberType.INT64):
+        return int
+    if key_type in (MemberType.FLOAT32, MemberType.FLOAT64):
+        return float
+    if key_type == MemberType.BOOLEAN:
+        return bool
+    return str
