@@ -4,15 +4,17 @@ Contains structs to build a :class:`DataRequest` instance: an object to describe
 the parameters needed for the query using only entity names and relationships.
 """
 
-import collections.abc
 from dataclasses import dataclass, field
 from itertools import chain
-from typing import Mapping, Optional, Sequence, Set, Tuple, Union
+from typing import Mapping, Optional, Set, Tuple, Union
 
 from typing_extensions import Literal, TypedDict
 
-from .models import (CutIntent, FilterCondition, FilterIntent,
-                     PaginationIntent, SortingIntent, TimeRestriction)
+from tesseract_olap.common import Array
+
+from .models import (CutIntent, DoubleFilterCondition, FilterIntent,
+                     PaginationIntent, SingleFilterCondition, SortingIntent,
+                     TimeRestriction)
 
 
 class DataRequestOptionalParams(TypedDict, total=False):
@@ -25,14 +27,14 @@ class DataRequestOptionalParams(TypedDict, total=False):
     This will give a better hint to the type checker when the user makes use of
     this interface.
     """
-    captions: Sequence[str]
-    cuts_exclude: Mapping[str, Sequence[Union[str, int]]]
-    cuts_include: Mapping[str, Sequence[Union[str, int]]]
-    filters: Mapping[str, FilterCondition]
+    captions: Array[str]
+    cuts_exclude: Mapping[str, Array[str]]
+    cuts_include: Mapping[str, Array[str]]
+    filters: Mapping[str, Union[SingleFilterCondition, DoubleFilterCondition]]
     locale: str
-    pagination: Tuple[int, int]
-    parents: Union[bool, Sequence[str], Set[str]]
-    properties: Sequence[str]
+    pagination: Union[str, Tuple[int, int]]
+    parents: Union[bool, Array[str]]
+    properties: Array[str]
     sorting: Tuple[str, Literal["asc", "desc"]]
     time: str
 
@@ -43,8 +45,8 @@ class DataRequestParams(DataRequestOptionalParams, total=True):
     Determines the expected params in a :class:`dict`, to use when creating a
     new :class:`DataRequest` object via the :func:`DataRequest.new` class method.
     """
-    drilldowns: Sequence[str]
-    measures: Sequence[str]
+    drilldowns: Array[str]
+    measures: Array[str]
 
 
 @dataclass(eq=False, order=False)
@@ -54,8 +56,10 @@ class DataRequest:
     All its properties are defined by strings of the names of the components
     from the schema.
     None of these parameters are verified during construction, so it's possible
-    the query results in to be invalid. The only purpose of this structure is
-    containing and passing this query intent.
+    for the query to be invalid; a subclass of :class:`backend.exceptions.BackendError`
+    will be raised in that case.
+    The only purpose of this structure is containing and passing over the query
+    intent to the internals.
 
     During a request, a :class:`Query` instance is constructed with objects from
     a schema, using parameters from this instance.
@@ -95,13 +99,13 @@ class DataRequest:
                 item: CutIntent.new(level=item,
                                     incl=cuts_include.get(item, []),
                                     excl=cuts_exclude.get(item, []))
-                for item in frozenset(
+                for item in set(
                     chain(cuts_include.keys(), cuts_exclude.keys())
                 )
             },
             "filters": {
-                item: FilterIntent(item, constraint)
-                for item, constraint in filters.items()
+                item: FilterIntent.new(field=item, condition=condition)
+                for item, condition in filters.items()
             },
             "captions": set(request.get("captions", [])),
             "properties": set(request.get("properties", [])),
@@ -111,23 +115,23 @@ class DataRequest:
         item = request.get("pagination", (0, 0))
         if isinstance(item, str):
             kwargs["pagination"] = PaginationIntent.from_str(item)
-        elif isinstance(item, collections.abc.Sequence):
+        elif isinstance(item, (list, tuple)):
             kwargs["pagination"] = PaginationIntent(*item)
 
         item = request.get("parents", False)
         if isinstance(item, (set, bool)):
             kwargs["parents"] = item
-        elif isinstance(item, collections.abc.Sequence):
+        elif isinstance(item, str):
+            kwargs["parents"] = {item}
+        elif isinstance(item, (list, tuple)):
             kwargs["parents"] = set(item)
 
         item = request.get("sorting")
         if isinstance(item, str):
             kwargs["sorting"] = SortingIntent.from_str(item)
-        elif isinstance(item, collections.abc.Sequence):
+        elif isinstance(item, (list, tuple)):
             kwargs["sorting"] = SortingIntent.new(*item)
 
-        # TODO: maybe these should be checked against Iterable/__getindex__?
-
         item = request.get("time")
         if isinstance(item, str):
             kwargs["time_restriction"] = TimeRestriction.from_str(item)
