@@ -5,9 +5,9 @@ for structs in the requests module.
 """
 
 from dataclasses import dataclass
-from typing import Any, Dict, Iterable, Sequence
+from typing import AsyncIterable
 
-from tesseract_olap.query.exceptions import EmptyResult
+from tesseract_olap.common import AnyDict, Array
 
 from .requests import DataRequest, MembersRequest
 
@@ -15,20 +15,13 @@ from .requests import DataRequest, MembersRequest
 @dataclass(eq=False, frozen=True, order=False)
 class DataResult:
     """Container class for results to :class:`DataRequest`."""
-    data: Iterable[Dict[str, Any]]
-    sources: Sequence[Dict[str, Any]]
+    data: AsyncIterable[AnyDict]
+    sources: Array[AnyDict]
     query: DataRequest
 
-    def raise_if_empty(self):
-        """Raises an :class:`EmptyResult` exception if the contained data has no
-        elements.
-        """
-        if next(iter(self.data), None) is None:
-            raise EmptyResult(self)
-
 
 @dataclass(eq=False, frozen=True, order=False)
 class MembersResult:
     """Container class for results to :class:`MembersRequest`."""
-    data: Iterable[Dict[str, Any]]
+    data: AsyncIterable[AnyDict]
     query: MembersRequest
